@@ -3,10 +3,13 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 import contextlib
-import io
 import os.path
 import traceback
 
+import six
+
+from pre_commit import five
+from pre_commit import output
 from pre_commit.errors import FatalError
 from pre_commit.store import Store
 
@@ -16,15 +19,26 @@ class PreCommitSystemExit(SystemExit):
     pass
 
 
-def _log_and_exit(msg, exc, formatted, print_fn=print):
-    error_msg = '{0}: {1}: {2}'.format(msg, type(exc).__name__, exc)
-    print_fn(error_msg)
-    print_fn('Check the log at ~/.pre-commit/pre-commit.log')
+def _to_bytes(exc):
+    try:
+        return bytes(exc)
+    except Exception:
+        return six.text_type(exc).encode('UTF-8')
+
+
+def _log_and_exit(msg, exc, formatted):
+    error_msg = b''.join((
+        five.to_bytes(msg), b': ',
+        five.to_bytes(type(exc).__name__), b': ',
+        _to_bytes(exc), b'\n',
+    ))
+    output.write(error_msg)
+    output.write_line('Check the log at ~/.pre-commit/pre-commit.log')
     store = Store()
     store.require_created()
-    with io.open(os.path.join(store.directory, 'pre-commit.log'), 'w') as log:
-        log.write(error_msg + '\n')
-        log.write(formatted + '\n')
+    with open(os.path.join(store.directory, 'pre-commit.log'), 'wb') as log:
+        output.write(error_msg, stream=log)
+        output.write_line(formatted, stream=log)
     raise PreCommitSystemExit(1)
 
 
