@@ -1,18 +1,30 @@
 from __future__ import unicode_literals
 
+from pre_commit.languages import docker
+from pre_commit.languages import golang
 from pre_commit.languages import node
 from pre_commit.languages import pcre
 from pre_commit.languages import python
 from pre_commit.languages import ruby
 from pre_commit.languages import script
+from pre_commit.languages import swift
 from pre_commit.languages import system
 
-# A language implements the following constant and two functions in its module:
+# A language implements the following constant and functions in its module:
 #
 # # Use None for no environment
 # ENVIRONMENT_DIR = 'foo_env'
 #
-# def install_environment(repo_cmd_runner, version='default'):
+# def get_default_version():
+#     """Return a value to replace the 'default' value for language_version.
+#
+#     return 'default' if there is no better option.
+#    """
+#
+# def healthy(repo_cmd_runner, language_version):
+#     """Return whether or not the environment is considered functional."""
+#
+# def install_environment(repo_cmd_runner, version, additional_dependencies):
 #     """Installs a repository in the given repository.  Note that the current
 #     working directory will already be inside the repository.
 #
@@ -36,11 +48,14 @@ from pre_commit.languages import system
 #     """
 
 languages = {
+    'docker': docker,
+    'golang': golang,
     'node': node,
     'pcre': pcre,
     'python': python,
     'ruby': ruby,
     'script': script,
+    'swift': swift,
     'system': system,
 }
 
