@@ -3,17 +3,20 @@ from __future__ import unicode_literals
 import contextlib
 import io
 import logging
-import os
 import os.path
 import sqlite3
 import tempfile
 
 from cached_property import cached_property
 
+import pre_commit.constants as C
 from pre_commit.prefixed_command_runner import PrefixedCommandRunner
 from pre_commit.util import clean_path_on_failure
 from pre_commit.util import cmd_output
+from pre_commit.util import copy_tree_to_path
 from pre_commit.util import cwd
+from pre_commit.util import no_git_env
+from pre_commit.util import resource_filename
 
 
 logger = logging.getLogger('pre_commit')
@@ -35,16 +38,6 @@ def _get_default_directory():
 class Store(object):
     get_default_directory = staticmethod(_get_default_directory)
 
-    class RepoPathGetter(object):
-        def __init__(self, repo, sha, store):
-            self._repo = repo
-            self._sha = sha
-            self._store = store
-
-        @cached_property
-        def repo_path(self):
-            return self._store.clone(self._repo, self._sha)
-
     def __init__(self, directory=None):
         if directory is None:
             directory = self.get_default_directory()
@@ -56,7 +49,7 @@ class Store(object):
         with io.open(os.path.join(self.directory, 'README'), 'w') as readme:
             readme.write(
                 'This directory is maintained by the pre-commit project.\n'
-                'Learn more: https://github.com/pre-commit/pre-commit\n'
+                'Learn more: https://github.com/pre-commit/pre-commit\n',
             )
 
     def _write_sqlite_db(self):
@@ -75,7 +68,7 @@ class Store(object):
                 '    ref CHAR(255) NOT NULL,'
                 '    path CHAR(255) NOT NULL,'
                 '    PRIMARY KEY (repo, ref)'
-                ');'
+                ');',
             )
 
         # Atomic file move
@@ -91,43 +84,59 @@ class Store(object):
 
     def require_created(self):
         """Require the pre-commit file store to be created."""
-        if self.__created:
-            return
+        if not self.__created:
+            self._create()
+            self.__created = True
 
-        self._create()
-        self.__created = True
-
-    def clone(self, url, sha):
-        """Clone the given url and checkout the specific sha."""
+    def _new_repo(self, repo, ref, make_strategy):
         self.require_created()
 
         # Check if we already exist
         with sqlite3.connect(self.db_path) as db:
             result = db.execute(
                 'SELECT path FROM repos WHERE repo = ? AND ref = ?',
-                [url, sha],
+                [repo, ref],
             ).fetchone()
             if result:
                 return result[0]
 
-        logger.info('Initializing environment for {0}.'.format(url))
+        logger.info('Initializing environment for {}.'.format(repo))
 
-        dir = tempfile.mkdtemp(prefix='repo', dir=self.directory)
-        with clean_path_on_failure(dir):
-            cmd_output('git', 'clone', '--no-checkout', url, dir)
-            with cwd(dir):
-                cmd_output('git', 'reset', sha, '--hard')
+        directory = tempfile.mkdtemp(prefix='repo', dir=self.directory)
+        with clean_path_on_failure(directory):
+            make_strategy(directory)
 
         # Update our db with the created repo
         with sqlite3.connect(self.db_path) as db:
             db.execute(
                 'INSERT INTO repos (repo, ref, path) VALUES (?, ?, ?)',
-                [url, sha, dir],
+                [repo, ref, directory],
             )
-        return dir
+        return directory
 
-    def get_repo_path_getter(self, repo, sha):
-        return self.RepoPathGetter(repo, sha, self)
+    def clone(self, repo, ref):
+        """Clone the given url and checkout the specific ref."""
+        def clone_strategy(directory):
+            cmd_output(
+                'git', 'clone', '--no-checkout', repo, directory,
+                env=no_git_env(),
+            )
+            with cwd(directory):
+                cmd_output('git', 'reset', ref, '--hard', env=no_git_env())
+                cmd_output(
+                    'git', 'submodule', 'update', '--init', '--recursive',
+                    env=no_git_env(),
+                )
+
+        return self._new_repo(repo, ref, clone_strategy)
+
+    def make_local(self, deps):
+        def make_local_strategy(directory):
+            copy_tree_to_path(resource_filename('empty_template'), directory)
+        return self._new_repo(
+            'local:{}'.format(','.join(sorted(deps))), C.LOCAL_REPO_VERSION,
+            make_local_strategy,
+        )
 
     @cached_property
     def cmd_runner(self):
